@@ -11,6 +11,8 @@ from paco.models.formatter import get_formatted_model_context, smart_join
 from paco.models.locations import get_parent_by_interface
 from paco.models.metrics import Monitorable, AlarmNotifications
 from paco.models.vocabulary import application_group_types, aws_regions
+from paco.models.logging import CloudWatchLogRetention
+from paco.models.iam import Role
 from zope.interface import implementer
 from zope.schema.fieldproperty import FieldProperty
 import troposphere
@@ -20,6 +22,7 @@ import troposphere.elasticloadbalancingv2
 import troposphere.elasticsearch
 import troposphere.s3
 import troposphere.secretsmanager
+import troposphere.ecs
 
 
 @implementer(schemas.IApplicationEngines)
@@ -181,6 +184,7 @@ class S3BucketPolicy(Parent):
     effect = FieldProperty(schemas.IS3BucketPolicy['effect'])
     principal = FieldProperty(schemas.IS3BucketPolicy['principal'])
     resource_suffix = FieldProperty(schemas.IS3BucketPolicy['resource_suffix'])
+    sid = FieldProperty(schemas.IS3BucketPolicy['sid'])
 
     def __init__(self, __parent__=None):
         super().__init__(__parent__)
@@ -448,9 +452,9 @@ class BlockDevice(Parent):
 
 @implementer(schemas.IBlockDeviceMapping)
 class BlockDeviceMapping(Parent):
-    device_name =  FieldProperty(schemas.IBlockDeviceMapping['device_name'])
-    ebs =  FieldProperty(schemas.IBlockDeviceMapping['ebs'])
-    virtual_name =  FieldProperty(schemas.IBlockDeviceMapping['virtual_name'])
+    device_name = FieldProperty(schemas.IBlockDeviceMapping['device_name'])
+    ebs = FieldProperty(schemas.IBlockDeviceMapping['ebs'])
+    virtual_name = FieldProperty(schemas.IBlockDeviceMapping['virtual_name'])
 
     @property
     def ebs_cfn(self):
@@ -473,34 +477,39 @@ class ASGRollingUpdatePolicy(Named):
     pause_time = FieldProperty(schemas.IASGRollingUpdatePolicy['pause_time'])
     wait_on_resource_signals = FieldProperty(schemas.IASGRollingUpdatePolicy['wait_on_resource_signals'])
 
+@implementer(schemas.IECSASGConfiguration)
+class ECSASGConfiguration(Named):
+    cluster = FieldProperty(schemas.IECSASGConfiguration['cluster'])
+
 @implementer(schemas.IASG)
 class ASG(ApplicationResource, Monitorable):
     title = "AutoScalingGroup"
-    cfn_init =  FieldProperty(schemas.IASG['cfn_init'])
-    desired_capacity =  FieldProperty(schemas.IASG['desired_capacity'])
-    desired_capacity_ignore_changes =  FieldProperty(schemas.IASG['desired_capacity_ignore_changes'])
-    min_instances =  FieldProperty(schemas.IASG['min_instances'])
-    max_instances =  FieldProperty(schemas.IASG['max_instances'])
-    associate_public_ip_address =  FieldProperty(schemas.IASG['associate_public_ip_address'])
-    cooldown_secs =  FieldProperty(schemas.IASG['cooldown_secs'])
-    ebs_optimized =  FieldProperty(schemas.IASG['ebs_optimized'])
-    health_check_type =  FieldProperty(schemas.IASG['health_check_type'])
-    health_check_grace_period_secs =  FieldProperty(schemas.IASG['health_check_grace_period_secs'])
-    eip =  FieldProperty(schemas.IASG['eip'])
-    instance_iam_role =  FieldProperty(schemas.IASG['instance_iam_role'])
-    instance_ami =  FieldProperty(schemas.IASG['instance_ami'])
-    instance_ami_ignore_changes =  FieldProperty(schemas.IASG['instance_ami_ignore_changes'])
-    instance_ami_type =  FieldProperty(schemas.IASG['instance_ami_type'])
-    instance_key_pair =  FieldProperty(schemas.IASG['instance_key_pair'])
-    instance_type =  FieldProperty(schemas.IASG['instance_type'])
-    segment =  FieldProperty(schemas.IASG['segment'])
-    termination_policies =  FieldProperty(schemas.IASG['termination_policies'])
-    security_groups =  FieldProperty(schemas.IASG['security_groups'])
+    cfn_init = FieldProperty(schemas.IASG['cfn_init'])
+    desired_capacity = FieldProperty(schemas.IASG['desired_capacity'])
+    desired_capacity_ignore_changes = FieldProperty(schemas.IASG['desired_capacity_ignore_changes'])
+    min_instances = FieldProperty(schemas.IASG['min_instances'])
+    max_instances = FieldProperty(schemas.IASG['max_instances'])
+    associate_public_ip_address = FieldProperty(schemas.IASG['associate_public_ip_address'])
+    cooldown_secs = FieldProperty(schemas.IASG['cooldown_secs'])
+    ebs_optimized = FieldProperty(schemas.IASG['ebs_optimized'])
+    health_check_type = FieldProperty(schemas.IASG['health_check_type'])
+    health_check_grace_period_secs = FieldProperty(schemas.IASG['health_check_grace_period_secs'])
+    eip = FieldProperty(schemas.IASG['eip'])
+    ecs = FieldProperty(schemas.IASG['ecs'])
+    instance_iam_role = FieldProperty(schemas.IASG['instance_iam_role'])
+    instance_ami = FieldProperty(schemas.IASG['instance_ami'])
+    instance_ami_ignore_changes = FieldProperty(schemas.IASG['instance_ami_ignore_changes'])
+    instance_ami_type = FieldProperty(schemas.IASG['instance_ami_type'])
+    instance_key_pair = FieldProperty(schemas.IASG['instance_key_pair'])
+    instance_type = FieldProperty(schemas.IASG['instance_type'])
+    segment = FieldProperty(schemas.IASG['segment'])
+    termination_policies = FieldProperty(schemas.IASG['termination_policies'])
+    security_groups = FieldProperty(schemas.IASG['security_groups'])
     target_groups = FieldProperty(schemas.IASG['target_groups'])
     load_balancers = FieldProperty(schemas.IASG['load_balancers'])
-    termination_policies =  FieldProperty(schemas.IASG['termination_policies'])
-    user_data_script =  FieldProperty(schemas.IASG['user_data_script'])
-    user_data_pre_script =  FieldProperty(schemas.IASG['user_data_pre_script'])
+    termination_policies = FieldProperty(schemas.IASG['termination_policies'])
+    user_data_script = FieldProperty(schemas.IASG['user_data_script'])
+    user_data_pre_script = FieldProperty(schemas.IASG['user_data_pre_script'])
     instance_monitoring = FieldProperty(schemas.IASG['instance_monitoring'])
     scaling_policy_cpu_average = FieldProperty(schemas.IASG['scaling_policy_cpu_average'])
     efs_mounts = FieldProperty(schemas.IASG['efs_mounts'])
@@ -522,6 +531,7 @@ class ASG(ApplicationResource, Monitorable):
         self.ebs_volume_mounts = []
         self.rolling_update_policy = ASGRollingUpdatePolicy('rolling_update_policy', self)
         self.launch_options = EC2LaunchOptions('launch_options', self)
+        self.instance_iam_role = Role('role', self)
 
     @property
     def instance_ami_type_family(self):
@@ -589,6 +599,409 @@ class ASG(ApplicationResource, Monitorable):
             return self.resource_name
         return None
 
+# ECS
+
+@implementer(schemas.IPortMapping)
+class PortMapping(Parent):
+    container_port = FieldProperty(schemas.IPortMapping['container_port'])
+    host_port = FieldProperty(schemas.IPortMapping['host_port'])
+    protocol = FieldProperty(schemas.IPortMapping['protocol'])
+
+    troposphere_props = troposphere.ecs.PortMapping.props
+    cfn_mapping = {
+        'ContainerPort': 'container_port',
+        'HostPort': 'host_port',
+        'Protocol': 'protocol',
+    }
+
+@implementer(schemas.IECSMountPoint)
+class ECSMountPoint(Parent):
+    "ECS TaskDefinition Mount Point"
+    container_path = FieldProperty(schemas.IECSMountPoint['container_path'])
+    read_only = FieldProperty(schemas.IECSMountPoint['read_only'])
+    source_volume = FieldProperty(schemas.IECSMountPoint['source_volume'])
+
+    troposphere_props = troposphere.ecs.MountPoint.props
+    cfn_mapping = {
+        'ContainerPath': 'container_path',
+        'SourceVolume': 'source_volume',
+        'ReadOnly': 'read_only',
+    }
+
+@implementer(schemas.IECSVolumesFrom)
+class ECSVolumesFrom(Parent):
+    "ECS VolumesFrom"
+    read_only = FieldProperty(schemas.IECSVolumesFrom['read_only'])
+    source_container = FieldProperty(schemas.IECSVolumesFrom['source_container'])
+
+    troposphere_props = troposphere.ecs.VolumesFrom.props
+    cfn_mapping = {
+        'SourceContainer': 'source_container',
+        'ReadOnly': 'read_only',
+    }
+
+@implementer(schemas.IECSLogging)
+class ECSLogging(Named, CloudWatchLogRetention):
+    driver = FieldProperty(schemas.IECSLogging['driver'])
+
+@implementer(schemas.IECSTaskDefinitionSecret)
+class ECSTaskDefinitionSecret(Parent, CFNExport):
+    name = FieldProperty(schemas.IECSTaskDefinitionSecret['name'])
+    value_from = FieldProperty(schemas.IECSTaskDefinitionSecret['value_from'])
+
+    troposphere_props = troposphere.ecs.Secret.props
+    cfn_mapping = {
+        'Name': 'name',
+        'ValueFrom': 'value_from'
+    }
+
+@implementer(schemas.IECSContainerDependency)
+class ECSContainerDependency(Parent):
+    container_name = FieldProperty(schemas.IECSContainerDependency['container_name'])
+    condition = FieldProperty(schemas.IECSContainerDependency['condition'])
+
+    troposphere_props = troposphere.ecs.ContainerDependency.props
+    cfn_mapping = {
+        'ContainerName': 'container_name',
+        'Condition': 'condition',
+    }
+
+@implementer(schemas.IDockerLabels)
+class DockerLabels(Named, dict):
+    pass
+
+@implementer(schemas.IECSHostEntry)
+class ECSHostEntry(Parent):
+    hostname = FieldProperty(schemas.IECSHostEntry['hostname'])
+    ip_address = FieldProperty(schemas.IECSHostEntry['ip_address'])
+
+@implementer(schemas.IECSHealthCheck)
+class ECSHealthCheck(Named):
+    command = FieldProperty(schemas.IECSHealthCheck['command'])
+    retries = FieldProperty(schemas.IECSHealthCheck['retries'])
+    timeout = FieldProperty(schemas.IECSHealthCheck['timeout'])
+    interval = FieldProperty(schemas.IECSHealthCheck['interval'])
+    start_period = FieldProperty(schemas.IECSHealthCheck['start_period'])
+
+    troposphere_props = troposphere.ecs.HealthCheck.props
+    cfn_mapping = {
+        'Command': 'command',
+        'Interval': 'interval',
+        'Retries': 'retries',
+        'StartPeriod': 'start_period',
+        'Timeout': 'timeout',
+    }
+
+@implementer(schemas.IECSUlimit)
+class ECSUlimit(Parent):
+    name = FieldProperty(schemas.IECSUlimit['name'])
+    hard_limit = FieldProperty(schemas.IECSUlimit['hard_limit'])
+    soft_limit = FieldProperty(schemas.IECSUlimit['soft_limit'])
+
+@implementer(schemas.IECSContainerDefinition)
+class ECSContainerDefinition(Named):
+    cpu = FieldProperty(schemas.IECSContainerDefinition['cpu'])
+    command = FieldProperty(schemas.IECSContainerDefinition['command'])
+    depends_on = FieldProperty(schemas.IECSContainerDefinition['depends_on'])
+    disable_networking = FieldProperty(schemas.IECSContainerDefinition['disable_networking'])
+    dns_search_domains = FieldProperty(schemas.IECSContainerDefinition['dns_search_domains'])
+    dns_servers = FieldProperty(schemas.IECSContainerDefinition['dns_servers'])
+    docker_labels = FieldProperty(schemas.IECSContainerDefinition['docker_labels'])
+    docker_security_options = FieldProperty(schemas.IECSContainerDefinition['docker_security_options'])
+    entry_point = FieldProperty(schemas.IECSContainerDefinition['entry_point'])
+    environment = FieldProperty(schemas.IECSContainerDefinition['environment'])
+    essential = FieldProperty(schemas.IECSContainerDefinition['essential'])
+    extra_hosts = FieldProperty(schemas.IECSContainerDefinition['extra_hosts'])
+    health_check = FieldProperty(schemas.IECSContainerDefinition['health_check'])
+    hostname = FieldProperty(schemas.IECSContainerDefinition['hostname'])
+    image = FieldProperty(schemas.IECSContainerDefinition['image'])
+    image_tag = FieldProperty(schemas.IECSContainerDefinition['image_tag'])
+    interactive = FieldProperty(schemas.IECSContainerDefinition['interactive'])
+    logging = FieldProperty(schemas.IECSContainerDefinition['logging'])
+    memory = FieldProperty(schemas.IECSContainerDefinition['memory'])
+    memory_reservation = FieldProperty(schemas.IECSContainerDefinition['memory_reservation'])
+    mount_points = FieldProperty(schemas.IECSContainerDefinition['mount_points'])
+    port_mappings = FieldProperty(schemas.IECSContainerDefinition['port_mappings'])
+    privileged = FieldProperty(schemas.IECSContainerDefinition['privileged'])
+    pseudo_terminal = FieldProperty(schemas.IECSContainerDefinition['pseudo_terminal'])
+    readonly_root_filesystem = FieldProperty(schemas.IECSContainerDefinition['readonly_root_filesystem'])
+    start_timeout = FieldProperty(schemas.IECSContainerDefinition['start_timeout'])
+    stop_timeout = FieldProperty(schemas.IECSContainerDefinition['stop_timeout'])
+    secrets = FieldProperty(schemas.IECSContainerDefinition['secrets'])
+    ulimits = FieldProperty(schemas.IECSContainerDefinition['ulimits'])
+    user = FieldProperty(schemas.IECSContainerDefinition['user'])
+    volumes_from = FieldProperty(schemas.IECSContainerDefinition['volumes_from'])
+    working_directory = FieldProperty(schemas.IECSContainerDefinition['working_directory'])
+
+    def __init__(self, name, parent):
+        super().__init__(name, parent)
+        self.port_mappings = []
+        self.volumes_from = []
+        self.mount_points = []
+        self.environment = []
+        self.secrets = []
+        self.depends_on = []
+        self.dns_search_domains = []
+        self.dns_servers = []
+        self.DockerLabels = DockerLabels('docker_labels', self)
+        self.ulimits = []
+
+    @property
+    def secrets_mappings_cfn(self):
+        return [
+            sm.cfn_export_dict for sm in self.secrets
+        ]
+
+    @property
+    def port_mappings_cfn(self):
+        return [
+            pm.cfn_export_dict for pm in self.port_mappings
+        ]
+
+    @property
+    def mount_points_cfn(self):
+        return [
+            mp.cfn_export_dict for mp in self.mount_points
+        ]
+
+    @property
+    def volumes_from_cfn(self):
+        return [
+            vf.cfn_export_dict for vf in self.volumes_from
+        ]
+
+    @property
+    def depends_on_cfn(self):
+        return [
+            dep.cfn_export_dict for dep in self.depends_on
+        ]
+
+    @property
+    def ulimits_cfn(self):
+        return [
+            ulimit.cfn_export_dict for ulimit in self.ulimits
+        ]
+
+    @property
+    def health_check_cfn(self):
+        if self.health_check != None:
+            return self.health_check.cfn_export_dict
+
+    @property
+    def ebsoptions_cfn(self):
+        if self.ebs_volumes != None:
+            return self.ebs_volumes.cfn_export_dict
+
+    troposphere_props = troposphere.ecs.ContainerDefinition.props
+    cfn_mapping = {
+        'Command': 'command',
+        'Cpu': 'cpu',
+        'DependsOn': 'depends_on_cfn',
+        'DisableNetworking': 'disable_networking',
+        'DnsSearchDomains': 'dns_search_domains',
+        'DnsServers': 'dns_servers',
+        'DockerLabels': 'docker_labels',
+        'DockerSecurityOptions': 'docker_security_options',
+        'EntryPoint': 'entry_point',
+        # 'Environment':  computed in template
+        'Essential': 'essential',
+        'ExtraHosts': 'extra_hosts',
+        # 'FirelensConfiguration': (FirelensConfiguration, False),
+        'HealthCheck': 'health_check_cfn',
+        'Hostname': 'hostname',
+        # 'Image': computed in the template
+        'Interactive': 'interactive',
+        # 'Links': ([basestring], False),
+        # 'LinuxParameters': (LinuxParameters, False),
+        # 'LogConfiguration': computed in template
+        'Memory': 'memory',
+        'MemoryReservation': 'memory_reservation',
+        'MountPoints': 'mount_points_cfn',
+        'Name': 'name',
+        'PortMappings': 'port_mappings_cfn',
+        'Privileged': 'privileged',
+        'PseudoTerminal': 'pseudo_terminal',
+        'ReadonlyRootFilesystem': 'readonly_root_filesystem',
+        # 'RepositoryCredentials': (RepositoryCredentials, False),
+        # 'ResourceRequirements': ([ResourceRequirement], False),
+        # 'ServiceRegistries': ([ServiceRegistry], False)
+        'Secrets': 'secrets_mappings_cfn',
+        'StartTimeout': 'start_timeout',
+        'StopTimeout': 'stop_timeout',
+        # 'SystemControls': ([SystemControl], False),
+        'Ulimits': 'ulimits_cfn',
+        'User': 'user',
+        'VolumesFrom': 'volumes_from_cfn',
+        'WorkingDirectory': 'working_directory',
+    }
+
+@implementer(schemas.IECSContainerDefinitions)
+class ECSContainerDefinitions(Named, dict):
+    pass
+
+@implementer(schemas.IECSTaskDefinitions)
+class ECSTaskDefinitions(Named, dict):
+    pass
+
+@implementer(schemas.IECSVolume)
+class ECSVolume(Parent):
+    "ECS Volume"
+    name = FieldProperty(schemas.IECSVolume['name'])
+
+    troposphere_props = troposphere.ecs.Volume.props
+    cfn_mapping = {
+        # 'DockerVolumeConfiguration': (DockerVolumeConfiguration, False),
+        'Name': 'name',
+        # 'Host': (Host, False),
+    }
+
+@implementer(schemas.IECSTaskDefinition)
+class ECSTaskDefinition(Named):
+    container_definitions = FieldProperty(schemas.IECSTaskDefinition['container_definitions'])
+    volumes = FieldProperty(schemas.IECSTaskDefinition['volumes'])
+    network_mode = FieldProperty(schemas.IECSTaskDefinition['network_mode'])
+
+    @property
+    def container_definitions_cfn(self):
+        return [
+            cd.cfn_export_dict for cd in self.container_definitions.values()
+        ]
+
+    @property
+    def volumes_cfn(self):
+        return [
+            v.cfn_export_dict for v in self.volumes
+        ]
+
+    troposphere_props = troposphere.ecs.TaskDefinition.props
+    cfn_mapping = {
+        'ContainerDefinitions': 'container_definitions_cfn',
+        # 'Cpu': (basestring, False),
+        # 'ExecutionRoleArn': created in the reseng
+        # 'Family': (basestring, False),
+        # 'InferenceAccelerators': ([InferenceAccelerator], False),
+        # 'IpcMode': (basestring, False),
+        # 'Memory': (basestring, False),
+        # 'NetworkMode': (basestring, False),
+        # 'PidMode': (basestring, False),
+        # 'PlacementConstraints': ([PlacementConstraint], False),
+        # 'ProxyConfiguration': (ProxyConfiguration, False),
+        # 'RequiresCompatibilities': ([basestring], False),
+        # 'Tags': (Tags, False),
+        # 'TaskRoleArn': (basestring, False),
+        'Volumes': 'volumes_cfn',
+    }
+
+    def __init__(self, name, parent):
+        super().__init__(name, parent)
+        self.volumes = []
+        self.container_definitions = ECSContainerDefinitions('container_definitions', self)
+
+@implementer(schemas.IECSLoadBalancer)
+class ECSLoadBalancer(Named):
+    container_name = FieldProperty(schemas.IECSLoadBalancer['container_name'])
+    container_port = FieldProperty(schemas.IECSLoadBalancer['container_port'])
+    target_group = FieldProperty(schemas.IECSLoadBalancer['target_group'])
+
+    troposphere_props = troposphere.ecs.LoadBalancer.props
+    cfn_mapping = {
+        'ContainerName': 'container_name',
+        'ContainerPort': 'container_port',
+        # 'LoadBalancerName': (basestring, False),
+        'TargetGroupArn': 'target_group',
+    }
+
+@implementer(schemas.IECSServicesContainer)
+class ECSServicesContainer(Named, dict):
+    pass
+
+@implementer(schemas.IECSService)
+class ECSService(Named):
+    deployment_controller = FieldProperty(schemas.IECSService['deployment_controller'])
+    desired_count = FieldProperty(schemas.IECSService['desired_count'])
+    task_definition = FieldProperty(schemas.IECSService['task_definition'])
+    load_balancers = FieldProperty(schemas.IECSService['load_balancers'])
+    hostname = FieldProperty(schemas.IECSService['hostname'])
+
+    def __init__(self, name, parent):
+        super().__init__(name, parent)
+        self.load_balancers = []
+
+    def resolve_ref(self, ref):
+        services = get_parent_by_interface(self, schemas.IECSServices)
+        return services.stack
+
+    @property
+    def load_balancers_cfn(self):
+        return [
+            lb.cfn_export_dict for lb in self.load_balancers
+        ]
+
+    @property
+    def deployment_controller_cfn(self):
+        return {"Type": self.deployment_controller.upper()}
+
+    troposphere_props = troposphere.ecs.Service.props
+    cfn_mapping = {
+        # 'Cluster': set in template
+        # 'DeploymentConfiguration': (DeploymentConfiguration, False),
+        'DeploymentController': 'deployment_controller_cfn',
+        'DesiredCount': 'desired_count',
+        # 'EnableECSManagedTags': (boolean, False),
+        # 'HealthCheckGracePeriodSeconds': (positive_integer, False),
+        # 'LaunchType': (launch_type_validator, False),
+        'LoadBalancers': 'load_balancers_cfn',
+        # 'NetworkConfiguration': (NetworkConfiguration, False),
+        # 'Role': (basestring, False),
+        # 'PlacementConstraints': ([PlacementConstraint], False),
+        # 'PlacementStrategies': ([PlacementStrategy], False),
+        # 'PlatformVersion': (basestring, False),
+        # 'PropagateTags': (basestring, False),
+        # 'SchedulingStrategy': (basestring, False),
+        # 'ServiceName': (basestring, False),
+        # 'ServiceRegistries': ([ServiceRegistry], False),
+        # 'Tags': (Tags, False),
+        'TaskDefinition': 'task_definition',
+    }
+
+@implementer(schemas.IECSServices)
+class ECSServices(Resource):
+    cluster = FieldProperty(schemas.IECSServices['cluster'])
+    services = FieldProperty(schemas.IECSServices['services'])
+    task_definitions = FieldProperty(schemas.IECSServices['task_definitions'])
+    service_discovery_namespace_name = FieldProperty(schemas.IECSServices['service_discovery_namespace_name'])
+    secrets_manager_access = FieldProperty(schemas.IECSServices['secrets_manager_access'])
+
+    def __init__(self, name, parent):
+        super().__init__(name, parent)
+        self.task_definitions = ECSTaskDefinitions('task_definitions', self)
+        self.services = ECSServicesContainer('services', self)
+
+@implementer(schemas.IECSCluster)
+class ECSCluster(Resource):
+
+    def resolve_ref(self, ref):
+        return self.stack
+
+# ECR: Elastic Container Repository
+@implementer(schemas.IECRRepository)
+class ECRRepository(Resource):
+    cross_account_access = FieldProperty(schemas.IECRRepository['cross_account_access'])
+    lifecycle_policy_registry_id = FieldProperty(schemas.IECRRepository['lifecycle_policy_registry_id'])
+    lifecycle_policy_text = FieldProperty(schemas.IECRRepository['lifecycle_policy_text'])
+    repository_name = FieldProperty(schemas.IECRRepository['repository_name'])
+    repository_policy = FieldProperty(schemas.IECRRepository['repository_policy'])
+
+    def __init__(self, name, parent):
+        super().__init__(name, parent)
+        self.cross_account_access = []
+
+    def resolve_ref(self, ref):
+        if ref.last_part == 'name':
+            return self.repository_name
+        return self.stack
+
+# EC2
 
 @implementer(schemas.IEC2)
 class EC2(ApplicationResource):
@@ -772,6 +1185,7 @@ class Lambda(ApplicationResource, Monitorable):
 class SNSTopicSubscription(Parent):
     protocol = FieldProperty(schemas.ISNSTopicSubscription['protocol'])
     endpoint = FieldProperty(schemas.ISNSTopicSubscription['endpoint'])
+    filter_policy = FieldProperty(schemas.ISNSTopicSubscription['filter_policy'])
 
 @implementer(schemas.ISNSTopic)
 class SNSTopic(Resource):
@@ -780,6 +1194,11 @@ class SNSTopic(Resource):
     display_name = FieldProperty(schemas.ISNSTopic['display_name'])
     subscriptions = FieldProperty(schemas.ISNSTopic['subscriptions'])
     cross_account_access = FieldProperty(schemas.ISNSTopic['cross_account_access'])
+    locations = FieldProperty(schemas.ISNSTopic["locations"])
+
+    def __init__(self, name, parent):
+        super().__init__(name, parent)
+        self.locations = []
 
     def resolve_ref(self, ref):
         return self.resolve_ref_obj.resolve_ref(ref)
@@ -1242,15 +1661,19 @@ class DeploymentPipelineSourceGitHub(DeploymentPipelineStageAction):
 @implementer(schemas.IDeploymentPipelineBuildCodeBuild)
 class DeploymentPipelineBuildCodeBuild(DeploymentPipelineStageAction):
     title = 'CodeBuild.Build'
-    deployment_environment = FieldProperty(schemas.IDeploymentPipelineBuildCodeBuild['deployment_environment'])
+    buildspec = FieldProperty(schemas.IDeploymentPipelineBuildCodeBuild['buildspec'])
     codebuild_image = FieldProperty(schemas.IDeploymentPipelineBuildCodeBuild['codebuild_image'])
     codebuild_compute_type = FieldProperty(schemas.IDeploymentPipelineBuildCodeBuild['codebuild_compute_type'])
-    timeout_mins = FieldProperty(schemas.IDeploymentPipelineBuildCodeBuild['timeout_mins'])
+    codecommit_repo_users = FieldProperty(schemas.IDeploymentPipelineBuildCodeBuild['codecommit_repo_users'])
+    deployment_environment = FieldProperty(schemas.IDeploymentPipelineBuildCodeBuild['deployment_environment'])
+    privileged_mode = FieldProperty(schemas.IDeploymentPipelineBuildCodeBuild['privileged_mode'])
     role_policies = FieldProperty(schemas.IDeploymentPipelineBuildCodeBuild['role_policies'])
+    timeout_mins = FieldProperty(schemas.IDeploymentPipelineBuildCodeBuild['timeout_mins'])
 
     def __init__(self, name, parent):
         super().__init__(name, parent)
         self.role_policies = []
+        self.codecommit_repo_users = []
 
 @implementer(schemas.IDeploymentPipelineDeployS3)
 class DeploymentPipelineDeployS3(DeploymentPipelineStageAction):
@@ -1284,7 +1707,7 @@ class CodeDeployMinimumHealthyHosts(Named):
 
 @implementer(schemas.IDeploymentPipelineDeployCodeDeploy)
 class DeploymentPipelineDeployCodeDeploy(DeploymentPipelineStageAction):
-    title = 'CodeDeploy.Source'
+    title = 'CodeDeploy.Deploy'
     auto_scaling_group = FieldProperty(schemas.IDeploymentPipelineDeployCodeDeploy['auto_scaling_group'])
     auto_rollback_enabled = FieldProperty(schemas.IDeploymentPipelineDeployCodeDeploy['auto_rollback_enabled'])
     minimum_healthy_hosts = FieldProperty(schemas.IDeploymentPipelineDeployCodeDeploy['minimum_healthy_hosts'])
@@ -1294,6 +1717,12 @@ class DeploymentPipelineDeployCodeDeploy(DeploymentPipelineStageAction):
     alb_target_group = FieldProperty(schemas.IDeploymentPipelineDeployCodeDeploy['alb_target_group'])
 
 
+@implementer(schemas.IDeploymentPipelineDeployECS)
+class DeploymentPipelineDeployECS(DeploymentPipelineStageAction):
+    title = 'ECS.Deploy'
+    cluster = FieldProperty(schemas.IDeploymentPipelineDeployECS['cluster'])
+    service = FieldProperty(schemas.IDeploymentPipelineDeployECS['service'])
+
 @implementer(schemas.IDeploymentPipelineSourceStage)
 class DeploymentPipelineSourceStage(Named, dict):
     pass
