@@ -19,11 +19,12 @@ from paco.models.networks import NetworkEnvironment, Environment, EnvironmentDef
     PrivateHostedZone, SecurityGroup, IngressRule, EgressRule, NATGateways, VPNGateways, Segments, \
     VPCPeerings, SecurityGroupSets, SecurityGroups
 from paco.models.project import VersionControl, Project, Credentials, SharedState, PacoWorkBucket
-from paco.models.applications import Application, ResourceGroups, ResourceGroup, RDS, ASG, \
-    Resource, Resources, LBApplication, TargetGroups, TargetGroup, Listeners, Listener, DNS, PortProtocol, EC2, S3Bucket, \
-    ApplicationS3Bucket, \
-    S3NotificationConfiguration, S3LambdaConfiguration, S3StaticWebsiteHosting, S3StaticWebsiteHostingRedirectRequests, \
-    S3BucketPolicy, AWSCertificateManager, ListenerRule, Lambda, LambdaEnvironment, LambdaVpcConfig, \
+from paco.models.applications import Application, ResourceGroups, ResourceGroup, RDS, \
+    ASG, ECSASGConfiguration, \
+    Resource, Resources, LBApplication, TargetGroups, TargetGroup, Listeners, Listener, DNS, PortProtocol, EC2, \
+    S3Bucket, ApplicationS3Bucket, S3NotificationConfiguration, S3LambdaConfiguration, \
+    S3StaticWebsiteHosting, S3StaticWebsiteHostingRedirectRequests, S3BucketPolicy, \
+    AWSCertificateManager, ListenerRule, Lambda, LambdaEnvironment, LambdaVpcConfig, \
     LambdaFunctionCode, LambdaVariable, SNSTopic, SNSTopicSubscription, \
     CloudFront, CloudFrontFactory, CloudFrontCustomErrorResponse, CloudFrontOrigin, CloudFrontCustomOriginConfig, \
     CloudFrontDefaultCacheBehavior, CloudFrontCacheBehavior, CloudFrontForwardedValues, CloudFrontCookies, CloudFrontViewerCertificate, \
@@ -32,12 +33,16 @@ from paco.models.applications import Application, ResourceGroups, ResourceGroup,
     DeploymentPipelineDeployStage, DeploymentPipelineSourceCodeCommit, DeploymentPipelineBuildCodeBuild, \
     DeploymentPipelineDeployCodeDeploy, DeploymentPipelineManualApproval, CodeDeployMinimumHealthyHosts, \
     DeploymentPipelineDeployS3, DeploymentPipelineLambdaInvoke, DeploymentPipelineSourceGitHub, \
-    DeploymentPipelinePacoCreateThenDeployImage, CodePipelineStage, CodePipelineStages, \
+    DeploymentPipelinePacoCreateThenDeployImage, DeploymentPipelineDeployECS, CodePipelineStage, CodePipelineStages, \
     EFS, EFSMount, ASGScalingPolicies, ASGScalingPolicy, ASGLifecycleHooks, ASGLifecycleHook, ASGRollingUpdatePolicy, EIP, \
     EBS, EBSVolumeMount, SecretsManager, SecretsManagerApplication, SecretsManagerGroup, SecretsManagerSecret, \
     GenerateSecretString, EC2LaunchOptions, DBParameterGroup, DBParameters, BlockDeviceMapping, BlockDevice, \
     CodeDeployApplication, CodeDeployDeploymentGroups, CodeDeployDeploymentGroup, DeploymentGroupS3Location, \
-    ElasticsearchDomain, ElasticsearchCluster, EBSOptions, ESAdvancedOptions
+    ElasticsearchDomain, ElasticsearchCluster, EBSOptions, ESAdvancedOptions, \
+    ECSContainerDefinition, ECSContainerDefinitions, ECSTaskDefinitions, ECSTaskDefinition, \
+    ECSLoadBalancer, ECSServicesContainer, ECSService, ECSCluster, ECSServices, PortMapping, ECSMountPoint, \
+    ECSVolumesFrom, ECSVolume, ECSLogging, ECRRepository, ECSTaskDefinitionSecret, ECSContainerDependency, \
+    DockerLabels, ECSHostEntry, ECSHealthCheck, ECSUlimit
 from paco.models.iot import IoTTopicRule, IoTTopicRuleAction, IoTTopicRuleLambdaAction, \
     IoTTopicRuleIoTAnalyticsAction, IoTAnalyticsPipeline, IoTPipelineActivities, IoTPipelineActivity, \
     IotAnalyticsStorage, Attributes, IoTDatasets, IoTDataset, DatasetTrigger, DatasetContentDeliveryRules, \
@@ -45,7 +50,7 @@ from paco.models.iot import IoTTopicRule, IoTTopicRuleAction, IoTTopicRuleLambda
     DatasetVariables, DatasetVariable, IoTPolicy, IoTVariables
 from paco.models.resources import EC2Resource, EC2KeyPairs, EC2KeyPair, S3Resource, S3Buckets, \
     Route53Resource, Route53HostedZone, Route53RecordSet, Route53HostedZoneExternalResource, Route53HealthCheck, \
-    CodeCommit, CodeCommitRepository, CodeCommitRepositoryGroup, CodeCommitUser, \
+    CodeCommit, CodeCommitRepository, CodeCommitRepositoryGroup, CodeCommitUser, CodeCommitUsers, \
     CloudTrailResource, CloudTrails, CloudTrail, \
     ApiGatewayRestApi, ApiGatewayMethods, ApiGatewayMethod, ApiGatewayStages, ApiGatewayStage, \
     ApiGatewayResources, ApiGatewayResource, ApiGatewayModels, ApiGatewayModel, \
@@ -54,7 +59,8 @@ from paco.models.resources import EC2Resource, EC2KeyPairs, EC2KeyPair, S3Resour
     IAMResource, IAMUser, IAMUsers, IAMUserPermission, IAMUserPermissions, IAMUserProgrammaticAccess, \
     IAMUserPermissionCodeCommitRepository, IAMUserPermissionCodeCommit, IAMUserPermissionAdministrator, \
     IAMUserPermissionCodeBuild, IAMUserPermissionCodeBuildResource, IAMUserPermissionCustomPolicy, \
-    SSMResource, SSMDocuments, SSMDocument
+    SSMResource, SSMDocuments, SSMDocument, ConfigResource, Config, \
+    SNS, Topics
 from paco.models.cfn_init import CloudFormationConfigSets, CloudFormationConfigurations, CloudFormationInitVersionedPackageSet, \
     CloudFormationInitPathOrUrlPackageSet, CloudFormationInitPackages, CloudFormationInitGroups, CloudFormationInitGroup, \
     CloudFormationInitUsers, CloudFormationInitUser, CloudFormationInitSources, CloudFormationInitFiles, \
@@ -64,7 +70,7 @@ from paco.models.cfn_init import CloudFormationConfigSets, CloudFormationConfigu
 from paco.models.backup import BackupPlanRule, BackupSelectionConditionResourceType, BackupPlanSelection, BackupPlan, \
     BackupPlans, BackupVault, BackupVaults
 from paco.models.events import EventsRule, EventTarget
-from paco.models.iam import IAM, ManagedPolicy, Role, Policy, AssumeRolePolicy, Statement
+from paco.models.iam import IAM, ManagedPolicy, Role, Policy, AssumeRolePolicy, Statement, Principal
 from paco.models.base import get_all_fields, most_specialized_interfaces, NameValuePair, RegionContainer, AccountRegions
 from paco.models.accounts import Account, AdminIAMUser
 from paco.models.references import Reference, PacoReference, FileReference
@@ -107,6 +113,7 @@ DEPLOYMENT_PIPELINE_STAGE_ACTION_CLASS_MAP = {
     'CodeBuild.Build': DeploymentPipelineBuildCodeBuild,
     'ManualApproval': DeploymentPipelineManualApproval,
     'CodeDeploy.Deploy': DeploymentPipelineDeployCodeDeploy,
+    'ECS.Deploy': DeploymentPipelineDeployECS,
     'S3.Deploy': DeploymentPipelineDeployS3,
     'Lambda.Invoke': DeploymentPipelineLambdaInvoke,
     'Paco.CreateThenDeployImage': DeploymentPipelinePacoCreateThenDeployImage,
@@ -120,34 +127,37 @@ IAM_USER_PERMISSIONS_CLASS_MAP = {
 }
 
 RESOURCES_CLASS_MAP = {
+    'ACM': AWSCertificateManager,
     'ApiGatewayRestApi': ApiGatewayRestApi,
     'ASG': ASG,
-    'ACM': AWSCertificateManager,
-    'RDS': RDS,
     'DBParameterGroup': DBParameterGroup,
-    'LBApplication': LBApplication,
-    'EC2': EC2,
-    'Lambda': Lambda,
-    'ManagedPolicy': ManagedPolicy,
-    'S3Bucket': ApplicationS3Bucket,
-    'SNSTopic': SNSTopic,
-    'CloudFront': CloudFront,
-    'RDSMysql': RDSMysql,
-    'RDSPostgresql': RDSPostgresql,
-    'ElastiCacheRedis': ElastiCacheRedis,
     'DeploymentPipeline': DeploymentPipeline,
-    'EFS': EFS,
-    'EIP': EIP,
-    'Route53HealthCheck': Route53HealthCheck,
-    'EventsRule': EventsRule,
-    'EBS': EBS,
-    'EBSVolumeMount': EBSVolumeMount,
+    'EC2': EC2,
+    'CloudFront': CloudFront,
     'CodeDeployApplication': CodeDeployApplication,
     'Dashboard': CloudWatchDashboard,
+    'EBS': EBS,
+    'EBSVolumeMount': EBSVolumeMount,
+    'ECSCluster': ECSCluster,
+    'ECSServices': ECSServices,
+    'ECRRepository': ECRRepository,
+    'EIP': EIP,
+    'EFS': EFS,
+    'ElastiCacheRedis': ElastiCacheRedis,
     'ElasticsearchDomain': ElasticsearchDomain,
+    'EventsRule': EventsRule,
     'IoTPolicy': IoTPolicy,
     'IoTTopicRule': IoTTopicRule,
     'IoTAnalyticsPipeline': IoTAnalyticsPipeline,
+    'Lambda': Lambda,
+    'LBApplication': LBApplication,
+    'ManagedPolicy': ManagedPolicy,
+    'RDS': RDS,
+    'RDSMysql': RDSMysql,
+    'RDSPostgresql': RDSPostgresql,
+    'Route53HealthCheck': Route53HealthCheck,
+    'S3Bucket': ApplicationS3Bucket,
+    'SNSTopic': SNSTopic,
 }
 
 SUB_TYPES_CLASS_MAP = {
@@ -158,6 +168,33 @@ SUB_TYPES_CLASS_MAP = {
         'version_control': ('direct_obj', VersionControl),
         'shared_state': ('direct_obj', SharedState),
     },
+    ECSServices: {
+        'task_definitions': ('container', (ECSTaskDefinitions, ECSTaskDefinition)),
+        'services': ('container', (ECSServicesContainer, ECSService)),
+    },
+    ECSService: {
+        'load_balancers': ('obj_list', ECSLoadBalancer),
+    },
+    ECSTaskDefinition: {
+        'container_definitions': ('container', (ECSContainerDefinitions, ECSContainerDefinition)),
+        'volumes': ('obj_list', ECSVolume),
+    },
+    ECSContainerDefinition: {
+        'depends_on': ('obj_list', ECSContainerDependency),
+        'docker_labels': ('dynamic_dict', DockerLabels),
+        'environment': ('obj_list', NameValuePair),
+        'extra_hosts': ('obj_list', ECSHostEntry),
+        'health_check': ('direct_obj', ECSHealthCheck),
+        'mount_points': ('obj_list', ECSMountPoint),
+        'port_mappings': ('obj_list', PortMapping),
+        'volumes_from': ('obj_list', ECSVolumesFrom),
+        'logging': ('direct_obj', ECSLogging),
+        'secrets': ('obj_list', ECSTaskDefinitionSecret),
+        'ulimits': ('obj_list', ECSUlimit),
+    },
+    ECRRepository: {
+        'repository_policy': ('direct_obj', Policy)
+    },
     IoTAnalyticsPipeline: {
         'channel_storage': ('direct_obj', IotAnalyticsStorage),
         'datastore_storage': ('direct_obj', IotAnalyticsStorage),
@@ -267,7 +304,8 @@ SUB_TYPES_CLASS_MAP = {
         'parameters': ('dynamic_dict', DBParameters)
     },
     DeploymentPipelineBuildCodeBuild: {
-        'role_policies': ('obj_list', Policy)
+        'role_policies': ('obj_list', Policy),
+        'codecommit_repo_users': ('str_list', PacoReference),
     },
     DeploymentPipelineDeployCodeDeploy: {
         'minimum_healthy_hosts': ('direct_obj', CodeDeployMinimumHealthyHosts)
@@ -350,7 +388,8 @@ SUB_TYPES_CLASS_MAP = {
         'viewer_certificate': ('direct_obj', CloudFrontViewerCertificate),
     },
     SNSTopic: {
-        'subscription': ('obj_list', SNSTopicSubscription),
+        'locations': ('obj_list', AccountRegions),
+        'subscriptions': ('obj_list', SNSTopicSubscription)
     },
     # Resource sub-objects
     CloudWatchAlarm: {
@@ -389,7 +428,8 @@ SUB_TYPES_CLASS_MAP = {
         'launch_options': ('direct_obj', EC2LaunchOptions),
         'cfn_init': ('obj_raw_config', CloudFormationInit),
         'block_device_mappings': ('obj_list', BlockDeviceMapping),
-        'rolling_update_policy': ('direct_obj', ASGRollingUpdatePolicy)
+        'rolling_update_policy': ('direct_obj', ASGRollingUpdatePolicy),
+        'ecs': ('direct_obj', ECSASGConfiguration),
     },
     EC2LaunchOptions: {
         'cfn_init_config_sets': ('str_list', zope.schema.TextLine)
@@ -423,12 +463,22 @@ SUB_TYPES_CLASS_MAP = {
     S3NotificationConfiguration: {
         'lambdas': ('obj_list', S3LambdaConfiguration)
     },
+    SNS: {
+        'default_locations': ('obj_list', AccountRegions),
+        'topics': ('container', (Topics, SNSTopic)),
+    },
     CloudTrailResource: {
         'trails': ('container', (CloudTrails, CloudTrail)),
     },
     CloudTrail: {
         'cloudwatchlogs_log_group': ('direct_obj', CloudWatchLogGroup),
     },
+    ConfigResource: {
+        'config': ('direct_obj',  Config),
+    },
+    Config: {
+        'locations': ('obj_list', AccountRegions),
+    },
 
     # monitoring and logging
     MonitorConfig: {
@@ -504,11 +554,16 @@ SUB_TYPES_CLASS_MAP = {
         'assume_role_policy': ('direct_obj', AssumeRolePolicy)
     },
     Policy: {
-        'statement': ('obj_list', Statement)
+        'statement': ('obj_list', Statement),
+    },
+    Principal: {
+        'aws': ('str_list', zope.schema.TextLine),
+        'service': ('str_list', zope.schema.TextLine)
     },
     Statement: {
         'action': ('str_list', zope.schema.TextLine),
-        'resource': ('str_list', zope.schema.TextLine)
+        'resource': ('str_list', zope.schema.TextLine),
+        'principal': ('direct_obj', Principal)
     },
     Lambda: {
         'environment': ('direct_obj', LambdaEnvironment),
@@ -548,13 +603,10 @@ SUB_TYPES_CLASS_MAP = {
     SSMDocument: {
         'locations': ('obj_list', AccountRegions),
     },
-    SNSTopic: {
-        'subscriptions': ('obj_list', SNSTopicSubscription)
-    },
     CodeCommit: {
     },
     CodeCommitRepository: {
-        'users': ('named_obj', CodeCommitUser)
+        'users': ('container', (CodeCommitUsers, CodeCommitUser))
     },
     IAMResource: {
         'users': ('container', (IAMUsers, IAMUser))
@@ -1231,7 +1283,7 @@ class ModelLoader():
 
     def __init__(self, config_folder, config_processor=None, warn=None):
         self.warn = warn
-        self.config_folder = config_folder
+        self.config_folder = pathlib.Path(config_folder)
         self.config_subdirs = {
             "monitor": self.instantiate_monitor_config,
             "accounts": self.instantiate_accounts,
@@ -1239,7 +1291,7 @@ class ModelLoader():
             "resource": self.instantiate_resources,
         }
         # Legacy directory names
-        if os.path.isdir(self.config_folder + os.sep + 'NetworkEnvironments'):
+        if os.path.isdir(self.config_folder / 'NetworkEnvironments'):
             self.config_subdirs = {
                 "MonitorConfig": self.instantiate_monitor_config,
                 "Accounts": self.instantiate_accounts,
@@ -1265,9 +1317,9 @@ Duplicate key \"{}\" found on line {} at column {}.
         """Read a YAML file"""
         # default is to load root project.yaml
         if sub_dir == '':
-            path = self.config_folder + os.sep + fname
+            path = self.config_folder / fname
         else:
-            path = self.config_folder + os.sep + sub_dir + os.sep + fname
+            path = self.config_folder / sub_dir / fname
         logger.debug("Loading YAML: %s" % (path))
 
         # Credential files must be secured. This seems hacky, is there a better way?
@@ -1303,24 +1355,26 @@ Duplicate key \"{}\" found on line {} at column {}.
         self.instantiate_project('project', self.read_yaml('', 'project.yaml'))
         self.project.paco_project_version = '{}.{}'.format(paco_project_version[0], paco_project_version[1])
 
-        if os.path.isfile(self.config_folder + os.sep + '.credentials.yaml'):
+        if os.path.isfile(self.config_folder / '.credentials.yaml'):
             self.instantiate_project('.credentials', self.read_yaml('', '.credentials.yaml'))
         for subdir, instantiate_method in self.config_subdirs.items():
-            for fname in os.listdir(self.config_folder + os.sep + subdir):
-                if fname.endswith('.yml') or fname.endswith('.yaml'):
-                    if fname.endswith('.yml'):
-                        name = fname[:-4]
-                    elif fname.endswith('.yaml'):
-                        name = fname[:-5]
-                    config = self.read_yaml(subdir, fname)
-                    instantiate_method(name, config, os.path.join(subdir, fname))
+            subpath = self.config_folder / subdir
+            if subpath.exists():
+                for fname in os.listdir(subpath):
+                    if fname.endswith('.yml') or fname.endswith('.yaml'):
+                        if fname.endswith('.yml'):
+                            name = fname[:-4]
+                        elif fname.endswith('.yaml'):
+                            name = fname[:-5]
+                        config = self.read_yaml(subdir, fname)
+                        instantiate_method(name, config, os.path.join(subdir, fname))
         self.instantiate_services()
         self.load_core_monitoring()
         self.yaml.restore_existing_constructors()
 
     def read_paco_project_version(self):
         "Reads <project>/paco-project-version.txt and returns a tuple (major,medium) version"
-        version_file = self.config_folder + os.sep + 'paco-project-version.txt'
+        version_file = self.config_folder / 'paco-project-version.txt'
         if not os.path.isfile(version_file):
             raise InvalidPacoProjectFile(
                 'You need a <project>/paco-project-version.txt file that declares your Paco project file format version.'
@@ -1591,13 +1645,13 @@ Duplicate key \"{}\" found on line {} at column {}.
         service_plugins = paco.models.services.list_service_plugins()
         services_dir_name = 'service'
         # Legacy directory name
-        if os.path.isdir(self.config_folder + os.sep + 'Services'):
+        if os.path.isdir(self.config_folder /'Services'):
             services_dir_name = 'Services'
-        services_dir = self.config_folder + os.sep + services_dir_name + os.sep
+        services_dir = self.config_folder / services_dir_name
         for plugin_name, plugin_module in service_plugins.items():
-            if os.path.isfile(services_dir + plugin_name + '.yml'):
+            if (services_dir / (plugin_name + '.yml')).is_file():
                 fname = plugin_name + '.yml'
-            elif os.path.isfile(services_dir + plugin_name + '.yaml'):
+            elif (services_dir / (plugin_name + '.yaml')).is_file():
                 fname = plugin_name + '.yaml'
             else:
                 continue
@@ -1606,7 +1660,7 @@ Duplicate key \"{}\" found on line {} at column {}.
                 config,
                 self.project,
                 self.monitor_config,
-                read_file_path=services_dir + fname
+                read_file_path=services_dir / fname
             )
             self.project['service'][plugin_name.lower()] = service
 
@@ -1652,6 +1706,12 @@ Duplicate key \"{}\" found on line {} at column {}.
 
         return snstopics
 
+    def instantiate_sns(self, config):
+        obj = SNS('sns', self.project.resource)
+        if config != None:
+            apply_attributes_from_config(obj, config, self.config_folder)
+        return obj
+
     def instantiate_cloudwatch_log_groups(self, config):
         cw_log_groups = CWLogGroups()
         self.project['cloudwatch_log_groups'] = cw_log_groups
@@ -1674,6 +1734,12 @@ Duplicate key \"{}\" found on line {} at column {}.
             apply_attributes_from_config(obj, config, self.config_folder)
         return obj
 
+    def instantiate_config(self, config):
+        obj = ConfigResource('config', self.project.resource)
+        if config != None:
+            apply_attributes_from_config(obj, config, self.config_folder)
+        return obj
+
     def instantiate_route53(self, config):
         obj = Route53Resource('route53', self.project.resource, config)
         if config != None:
