@@ -20,6 +20,25 @@ from paco.models import references
 class GlobalResources(Named, dict):
     "Global Resources"
 
+@implementer(schemas.ITopics)
+class Topics(Named, dict):
+    pass
+
+class Computed(Named, dict):
+    pass
+
+@implementer(schemas.ISNS)
+class SNS(Named, dict):
+    default_locations = FieldProperty(schemas.ISNS['default_locations'])
+    topics = FieldProperty(schemas.ISNS['topics'])
+
+    def __init__(self, name, parent):
+        super().__init__(name, parent)
+        self.computed = Computed('computed', self)
+
+    def resolve_ref(self, ref):
+        return self.resolve_ref_obj.resolve_ref(ref)
+
 @implementer(schemas.IApiGatewayMethods)
 class ApiGatewayMethods(Named, dict):
     pass
@@ -400,10 +419,15 @@ class Route53HealthCheck(Resource):
         else:
             return self.health_check_type
 
+@implementer(schemas.ICodeCommitUsers)
+class CodeCommitUsers(Named, dict):
+    pass
+
 @implementer(schemas.ICodeCommitUser)
-class CodeCommitUser():
+class CodeCommitUser(Named):
     username = FieldProperty(schemas.ICodeCommitUser["username"])
     public_ssh_key = FieldProperty(schemas.ICodeCommitUser["public_ssh_key"])
+    permissions = FieldProperty(schemas.ICodeCommitUser["permissions"])
 
 @implementer(schemas.ICodeCommitRepository)
 class CodeCommitRepository(Named, Deployable):
@@ -414,6 +438,10 @@ class CodeCommitRepository(Named, Deployable):
     users = FieldProperty(schemas.ICodeCommitRepository["users"])
     external_resource = FieldProperty(schemas.ICodeCommitRepository["external_resource"])
 
+    def __init__(self, name, parent):
+        super().__init__(name, parent)
+        self.user = CodeCommitUsers('users', self)
+
 @implementer(schemas.ICodeCommitRepositoryGroup)
 class CodeCommitRepositoryGroup(Named, dict):
     pass
@@ -457,6 +485,35 @@ class CodeCommit(Named, dict):
     def resolve_ref(self, ref):
         return self.resolve_ref_obj.resolve_ref(ref)
 
+@implementer(schemas.IConfig)
+class Config(Resource):
+    delivery_frequency = FieldProperty(schemas.IConfig["delivery_frequency"])
+    global_resources_region = FieldProperty(schemas.IConfig["global_resources_region"])
+    locations = FieldProperty(schemas.IConfig["locations"])
+    s3_bucket_logs_account = FieldProperty(schemas.IConfig["s3_bucket_logs_account"])
+
+    def __init__(self, name, __parent__):
+        super().__init__(name, __parent__)
+        self.locations = []
+
+    def get_accounts(self):
+        """
+        Resolve the locations field for all accounts.
+        If locations is empty, then all accounts are returned.
+        """
+        if self.locations == []:
+            return project['accounts'].values()
+        accounts = []
+        project = get_parent_by_interface(self, schemas.IProject)
+        for location in self.locations:
+            account = references.get_model_obj_from_ref(location.account, project)
+            accounts.append(account)
+        return accounts
+
+@implementer(schemas.IConfigResource)
+class ConfigResource(Named):
+    config = FieldProperty(schemas.IConfigResource["config"])
+
 @implementer(schemas.ICloudTrail)
 class CloudTrail(Resource):
     type = 'CloudTrail'
@@ -495,9 +552,8 @@ class CloudTrailResource(Named):
         self.trails = CloudTrails('trails', self)
 
     def resolve_ref(self, ref):
-        # Needed to satisfy paco.models.resolve_ref when it walks
-        # to an S3 Bucket within a CloudTrail
-        return None
+        return self.resolve_ref_obj.resolve_ref(ref)
+
 
 @implementer(schemas.IIAMUserProgrammaticAccess)
 class IAMUserProgrammaticAccess(Deployable):
