@@ -132,6 +132,16 @@ class Testpacodemo(BaseTestModelLoader):
         assert cfn_init.parameters['TestString'] == 'catdog'
         assert cfn_init.parameters['TestNumber'] == 10
 
+        # Test IAM Role
+        assert asg.instance_iam_role.enabled == True
+        assert asg.instance_iam_role.name == 'instance_iam_role'
+
+        norole_asg = demo_env.applications['app'].groups['site'].resources['norole']
+        assert norole_asg.instance_iam_role.enabled == False
+
+        # ECS
+        asg = demo_env.applications['app'].groups['container'].resources['ecs_asg']
+        assert asg.ecs.cluster, 'paco.ref netenv.pacodemo.demo.us-west-2.applications.app.groups.container.resources.ecs_cluster'
 
     def test_netenv_refs(self):
         demo_env = self.project['netenv']['pacodemo']['demo']['us-west-2']
@@ -301,6 +311,24 @@ class Testpacodemo(BaseTestModelLoader):
         assert bob.subscriptions[6].endpoint, 'arn:aws:sqs:us-east-2:444455556666:queue1'
         assert bob.subscriptions[7].endpoint, 'arn:aws:lambda:us-east-1:123456789012:function:my-function'
 
+    def test_sns(self):
+        sns = self.project['resource']['sns']
+        assert len(sns.default_locations), 1
+        assert len(sns.topics), 2
+
+    def test_ecs(self):
+        demo_env = self.project['netenv']['pacodemo']['demo']['us-west-2']
+        cluster = demo_env['applications']['app'].groups['container'].resources['ecs_cluster']
+        assert schemas.IECSCluster.providedBy(cluster)
+        ecs_service_config = demo_env['applications']['app'].groups['container'].resources['ecs_services']
+        assert ecs_service_config.services['simple_app'].desired_count, 2
+        assert ecs_service_config.services['simple_app'].load_balancers[0].container_name, 'hello'
+        assert ecs_service_config.task_definitions['hello_web'].container_definitions['hello'].cpu, 10
+        assert ecs_service_config.task_definitions['hello_web'].container_definitions['hello'].depends_on[0].condition, 'START'
+        assert ecs_service_config.task_definitions['hello_web'].container_definitions['hello'].health_check.retries, 5
+        assert ecs_service_config.task_definitions['hello_web'].container_definitions['hello'].ulimits[0].hard_limit, 1000
+        assert ecs_service_config.task_definitions['hello_web'].container_definitions['hello'].user, 'www-data'
+
     def test_lambda(self):
         demo_env = self.project['netenv']['pacodemo']['demo']['us-west-2']
         lmbda = demo_env['applications']['notification'].groups['lambda'].resources['function']
@@ -328,6 +356,11 @@ class Testpacodemo(BaseTestModelLoader):
         assert trail.cloudwatchlogs_log_group.log_group_name, 'CloudTrail'
         assert trail.cloudwatchlogs_log_group.expire_events_after_days, '14'
 
+    def test_config(self):
+        config = self.project['resource']['config'].config
+        assert config.global_resources_region == 'us-west-2'
+        assert config.delivery_frequency == 'Six_Hours'
+
     def test_api_gateway_rest_api(self):
         demo_env = self.project['netenv']['pacodemo']['demo']['us-west-2']
         api_gra = demo_env['applications']['app'].groups['restapi'].resources['api_gateway_rest_api']
